@@ -1,9 +1,11 @@
 # -*- coding: utf-8 -*-
 import os
+import re
 import shutil
 import sys
 from datetime import date
 from pathlib import Path
+import platform
 from tempfile import NamedTemporaryFile
 from typing import Optional
 from urllib.error import HTTPError
@@ -12,6 +14,7 @@ import requests
 import yaml
 from dotty_dict import Dotty
 from progress.bar import IncrementalBar
+from semver import VersionInfo
 
 from ddb import __version__
 from ddb.action import Action, InitializableAction
@@ -78,6 +81,68 @@ def print_version(github_repository, silent=False):
     print(get_table_display(blocks))
 
 
+_version_re = re.compile(
+    r"""[vV]?
+        (?P<major>0|[1-9]\d*)
+        (\.
+        (?P<minor>0|[1-9]\d*)
+        (\.
+            (?P<patch>0|[1-9]\d*)
+        )?
+        )?
+    """,
+    re.VERBOSE,
+)
+
+
+def coerce_version(version):
+    """
+    Convert an incomplete version string into a semver-compatible VersionInfo
+    object
+
+    * Tries to detect a "basic" version string (``major.minor.patch``).
+    * If not enough components can be found, missing components are
+        set to zero to obtain a valid semver version.
+
+    :param str version: the version string to convert
+    :return: a tuple with a :class:`VersionInfo` instance (or ``None``
+        if it's not a version) and the rest of the string which doesn't
+        belong to a basic version.
+    :rtype: tuple(:class:`VersionInfo` | None, str)
+    """
+    match = _version_re.search(version)
+    if not match:
+        return None, version
+
+    ver = {
+        key: 0 if value is None else value for key, value in match.groupdict().items()
+    }
+
+    ver = VersionInfo(**ver)
+    rest = match.string[match.end():]
+    return ver, rest
+
+
+def is_version_greater(reference, version):
+    """
+    Check if version is greater than reference.
+    :param reference:
+    :param version:
+    :return:
+    """
+    return coerce_version(reference) < coerce_version(version)
+
+
+def is_version_greater_or_equal(reference, version):
+    """
+    Check if version is greater or equal than reference.
+    :param reference:
+    :param version:
+    :return:
+    """
+    return coerce_version(reference) < coerce_version(version)
+
+
 def check_for_update(github_repository: str, output=False, details=False):
     """
     Check if a new version is available on github.
@@ -88,7 +153,7 @@ def check_for_update(github_repository: str, output=False, details=False):
     """
     last_release = get_latest_release_version(github_repository)
 
-    if last_release and get_current_version() < last_release:
+    if last_release and is_version_greater(get_current_version(), last_release):
         if output:
             blocks = [_build_update_header(last_release)]
             if details:
@@ -154,9 +219,26 @@ def get_binary_destination_path(binary_path: str):
             and Path(binary_path).read_text().startswith("#!/usr/bin/env python"):
         # Avoid removing main source file when running on development.
         binary_path = binary_path[:-3] + ".bin"
+
+    for qualifier in ['-linux', '-macos', '-windows']:
+        binary_path = binary_path.replace(qualifier, '')
     return binary_path
 
 
+def get_binary_remote_name():
+    """
+    Get binary remote name
+    :return:
+    """
+    if platform.system() == 'Windows':
+        return 'ddb-windows.exe'
+    if platform.system() == 'Darwin':
+        return 'ddb-macos'
+    if platform.system() == 'Linux':
+        return 'ddb-linux'
+    return None
+
+
 class FeaturesAction(Action):
     """
     Display all features
@@ -435,7 +517,7 @@ class CheckRequiredVersion(Action):
             required_version = config.data.get('core.required_version')
             if not required_version:
                 return
-            if required_version > get_current_version():
+            if is_version_greater_or_equal(get_current_version(), required_version):
                 update_tip = _build_update_tip()
                 if update_tip:
                     update_tip = ' ' + update_tip
@@ -496,7 +578,11 @@ class SelfUpdateAction(Action):
         if not os.access(binary_path, os.W_OK):
             raise PermissionError(f"You don't have permission to write on ddb binary file. ({binary_path})")
 
-        remote_filename = 'ddb.exe' if os.name == 'nt' else 'ddb'
+        remote_filename = get_binary_remote_name()
+        if not remote_filename:
+            print('ddb is running from a platform that doesn\'t support binary package mode.')
+            return
+
         url = 'https://github.com/{}/releases/download/v{}/{}'.format(github_repository, version, remote_filename)
 
         progress_bar = None
