@@ -1,36 +1,20 @@
 import os
-from typing import Iterable
 
 import pytest
 from _pytest.capture import CaptureFixture
 
 from ddb.__main__ import load_registered_features, register_actions_in_event_bus
-from ddb.binary import binaries, Binary
+from ddb.binary import binaries
+from ddb.binary.binary import DefaultBinary
 from ddb.config import config
 from ddb.feature import features
 from ddb.feature.core import CoreFeature
 from ddb.feature.docker.binaries import DockerBinary
 from ddb.feature.run import RunFeature, RunAction
+from ddb.feature.shell import ShellFeature
 from ddb.utils.docker import DockerUtils
 
-
-class BinaryMock(Binary):
-    def pre_execute(self):
-        return True
-
-    def __init__(self, name, *command):
-        self._name = name
-        self._command = command
-
-    def command(self, *args) -> Iterable[str]:
-        return self._command
-
-    @property
-    def name(self) -> str:
-        return self._name
-
-    def is_same(self, binary) -> bool:
-        return self.command() == binary.command()
+docker_compose_bin = "docker-compose" if os.name != "nt" else "docker-compose.exe"
 
 
 class TestRunFeature:
@@ -38,7 +22,9 @@ class TestRunFeature:
         project_loader("empty")
 
         features.register(RunFeature())
+        features.register(ShellFeature())
         load_registered_features()
+        register_actions_in_event_bus(True)
 
         action = RunAction()
         action.execute()
@@ -48,7 +34,9 @@ class TestRunFeature:
 
         features.register(CoreFeature())
         features.register(RunFeature())
+        features.register(ShellFeature())
         load_registered_features()
+        register_actions_in_event_bus(True)
 
         action = RunAction()
         action.execute()
@@ -58,7 +46,9 @@ class TestRunFeature:
 
         features.register(CoreFeature())
         features.register(RunFeature())
+        features.register(ShellFeature())
         load_registered_features()
+        register_actions_in_event_bus(True)
 
         action = RunAction()
 
@@ -70,10 +60,11 @@ class TestRunFeature:
 
         features.register(CoreFeature())
         features.register(RunFeature())
+        features.register(ShellFeature())
         load_registered_features()
         register_actions_in_event_bus(True)
 
-        binaries.register(BinaryMock("test", "some", "command"))
+        binaries.register(DefaultBinary("test", ["some", "command"]))
 
         action = RunAction()
         action.run("test")
@@ -88,6 +79,7 @@ class TestRunFeature:
 
         features.register(CoreFeature())
         features.register(RunFeature())
+        features.register(ShellFeature())
         load_registered_features()
         register_actions_in_event_bus(True)
 
@@ -98,7 +90,7 @@ class TestRunFeature:
 
         read = capsys.readouterr()
 
-        assert read.out == "docker-compose run --rm --workdir=/app/. service\n"
+        assert read.out == docker_compose_bin + " run --rm --workdir=/app/. service\n"
 
     def test_run_docker_binary_workdir(self, project_loader, capsys: CaptureFixture):
         project_loader("empty")
@@ -106,6 +98,7 @@ class TestRunFeature:
 
         features.register(CoreFeature())
         features.register(RunFeature())
+        features.register(ShellFeature())
         load_registered_features()
         register_actions_in_event_bus(True)
 
@@ -116,13 +109,40 @@ class TestRunFeature:
 
         read = capsys.readouterr()
 
-        assert read.out == "docker-compose run --rm --workdir=/app/sub service\n"
+        assert read.out == docker_compose_bin + " run --rm --workdir=/app/sub service\n"
+
+    def test_run_docker_binary_workdir_outside(self, project_loader, capsys: CaptureFixture):
+        project_loader("outside")
+        config.cwd = os.path.join(config.paths.project_home, "../outside")
+
+        features.register(CoreFeature())
+        features.register(RunFeature())
+        features.register(ShellFeature())
+        load_registered_features()
+        register_actions_in_event_bus(True)
+
+        binaries.register(DockerBinary("bin", "service", "/app"))
+
+        action = RunAction()
+        action.run("bin")
+
+        read = capsys.readouterr()
+
+        cwd = config.cwd if config.cwd else os.getcwd()
+        real_cwd = os.path.realpath(cwd)
+
+        assert read.out == docker_compose_bin + " -f ../project/docker-compose.yml " \
+                                                "run --rm "\
+                                                f"--volume={real_cwd}:/app " \
+                                                "--workdir=/app " \
+                                                "service\n"
 
     def test_run_docker_binary_exe(self, project_loader, capsys: CaptureFixture):
         project_loader("exe")
 
         features.register(CoreFeature())
         features.register(RunFeature())
+        features.register(ShellFeature())
         load_registered_features()
         register_actions_in_event_bus(True)
 
@@ -139,6 +159,31 @@ class TestRunFeature:
 
         read = capsys.readouterr()
 
-        assert read.out == "docker-compose exec web echo\n"
+        assert read.out == docker_compose_bin + " exec web echo\n"
 
-        assert DockerUtils.is_container_up('web')
\ No newline at end of file
+        assert DockerUtils.is_container_up('web')
+
+    def test_run_binary_condition(self, project_loader, capsys: CaptureFixture):
+        project_loader("empty")
+
+        features.register(CoreFeature())
+        features.register(RunFeature())
+        features.register(ShellFeature())
+        load_registered_features()
+        register_actions_in_event_bus(True)
+
+        binary1 = DockerBinary("npm", "node1")
+        binary2 = DockerBinary("npm", "node2")
+        default_binary = DefaultBinary("npm", ["npm"])
+        binary4 = DockerBinary("npm", "node4")
+        binary_invalid_condition = DockerBinary("npm", "node5", condition="'another-value' in args")
+        binary_valid_condition = DockerBinary("npm", "node6", condition="'some-value' in args")
+
+        for bin in (binary1, binary2, default_binary, binary4, binary_invalid_condition, binary_valid_condition):
+            binaries.register(bin)
+
+        action = RunAction()
+        action.run("npm", "some-value")
+
+        read = capsys.readouterr()
+        assert "node6" in read.out
